@@ -1,8 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -53,6 +51,7 @@ static inline void _phpdbg_break_mapping(int id, HashTable *table) /* {{{ */
 	zend_hash_index_del(&PHPDBG_G(bp)[PHPDBG_BREAK_MAP], (id))
 
 #define PHPDBG_BREAK_INIT(b, t) do {\
+	memset(&b, 0, sizeof(b)); \
 	b.id = PHPDBG_G(bp_count)++; \
 	b.type = t; \
 	b.disabled = 0;\
@@ -136,7 +135,7 @@ PHPDBG_API void phpdbg_export_breakpoints_to_string(char **str) /* {{{ */
 
 					switch (brake->type) {
 						case PHPDBG_BREAK_FILE: {
-							zend_string *filename = php_addcslashes(zend_string_init(((phpdbg_breakfile_t*)brake)->filename, strlen(((phpdbg_breakfile_t*)brake)->filename), 0), 1, "\\\"\n", 3);
+							zend_string *filename = php_addcslashes_str(((phpdbg_breakfile_t*)brake)->filename, strlen(((phpdbg_breakfile_t*)brake)->filename), "\\\"\n", 3);
 							phpdbg_asprintf(&new_str,
 								"%sbreak \"%s\":%lu\n", *str,
 								ZSTR_VAL(filename),
@@ -173,7 +172,7 @@ PHPDBG_API void phpdbg_export_breakpoints_to_string(char **str) /* {{{ */
 						} break;
 
 						case PHPDBG_BREAK_FILE_OPLINE: {
-							zend_string *filename = php_addcslashes(zend_string_init(((phpdbg_breakopline_t*)brake)->class_name, strlen(((phpdbg_breakopline_t*)brake)->class_name), 0), 1, "\\\"\n", 3);
+							zend_string *filename = php_addcslashes_str(((phpdbg_breakopline_t*)brake)->class_name, strlen(((phpdbg_breakopline_t*)brake)->class_name), "\\\"\n", 3);
 							phpdbg_asprintf(&new_str,
 								"%sbreak \"%s\":#%llu\n", *str,
 								ZSTR_VAL(filename),
@@ -192,6 +191,24 @@ PHPDBG_API void phpdbg_export_breakpoints_to_string(char **str) /* {{{ */
 
 							if (conditional->paramed) {
 								switch (conditional->param.type) {
+		                            case NUMERIC_FUNCTION_PARAM:
+		                                phpdbg_asprintf(&new_str,
+		                                    "%sbreak at %s#%ld if %s\n",
+		                                    *str, conditional->param.str, conditional->param.num, conditional->code);
+		                            break;
+
+		                            case NUMERIC_METHOD_PARAM:
+		                                phpdbg_asprintf(&new_str,
+		                                    "%sbreak at %s::%s#%ld if %s\n",
+		                                    *str, conditional->param.method.class, conditional->param.method.name, conditional->param.num, conditional->code);
+		                            break;
+
+		                            case ADDR_PARAM:
+		                                phpdbg_asprintf(&new_str,
+		                                    "%sbreak at 0X%lx if %s\n",
+		                                    *str, conditional->param.addr, conditional->code);
+		                            break;
+
 									case STR_PARAM:
 										phpdbg_asprintf(&new_str,
 											"%sbreak at %s if %s\n", *str, conditional->param.str, conditional->code);
@@ -205,7 +222,7 @@ PHPDBG_API void phpdbg_export_breakpoints_to_string(char **str) /* {{{ */
 									break;
 
 									case FILE_PARAM: {
-										zend_string *filename = php_addcslashes(zend_string_init(conditional->param.file.name, strlen(conditional->param.file.name), 0), 1, "\\\"\n", 3);
+										zend_string *filename = php_addcslashes_str(conditional->param.file.name, strlen(conditional->param.file.name), "\\\"\n", 3);
 										phpdbg_asprintf(&new_str,
 											"%sbreak at \"%s\":%lu if %s\n", *str,
 											ZSTR_VAL(filename), conditional->param.file.line,
@@ -232,7 +249,7 @@ PHPDBG_API void phpdbg_export_breakpoints_to_string(char **str) /* {{{ */
 		} ZEND_HASH_FOREACH_END();
 	}
 
-	if (!(*str)[0]) {
+	if ((*str) && !(*str)[0]) {
 		*str = NULL;
 	}
 } /* }}} */
@@ -811,7 +828,22 @@ static inline void phpdbg_create_conditional_break(phpdbg_breakcond_t *brake, co
 {
 	phpdbg_breakcond_t new_break;
 	uint32_t cops = CG(compiler_options);
-	zval pv;
+	zend_string *bp_code;
+
+	switch (param->type) {
+	    case STR_PARAM:
+		case NUMERIC_FUNCTION_PARAM:
+		case METHOD_PARAM:
+		case NUMERIC_METHOD_PARAM:
+		case FILE_PARAM:
+		case ADDR_PARAM:
+		    /* do nothing */
+		break;
+
+		default:
+			phpdbg_error("eval", "type=\"invalidparameter\"", "Invalid parameter type for conditional breakpoint");
+			return;
+	}
 
 	PHPDBG_BREAK_INIT(new_break, PHPDBG_BREAK_COND);
 	new_break.hash = hash;
@@ -820,6 +852,20 @@ static inline void phpdbg_create_conditional_break(phpdbg_breakcond_t *brake, co
 		new_break.paramed = 1;
 		phpdbg_copy_param(
 			param, &new_break.param);
+	    if (new_break.param.type == FILE_PARAM ||
+	        new_break.param.type == NUMERIC_FILE_PARAM) {
+	        char realpath[MAXPATHLEN];
+
+	        if (VCWD_REALPATH(new_break.param.file.name, realpath)) {
+	            efree(new_break.param.file.name);
+
+	            new_break.param.file.name = estrdup(realpath);
+	        } else {
+	            phpdbg_error("eval", "type=\"invalidparameter\"", "Invalid file for conditional break %s", new_break.param.file.name);
+	            phpdbg_clear_param(&new_break.param);
+	            return;
+	        }
+	    }
 	} else {
 		new_break.paramed = 0;
 	}
@@ -831,16 +877,10 @@ static inline void phpdbg_create_conditional_break(phpdbg_breakcond_t *brake, co
 	new_break.code = estrndup(expr, expr_len);
 	new_break.code_len = expr_len;
 
-	Z_STR(pv) = zend_string_alloc(expr_len + sizeof("return ;") - 1, 0);
-	memcpy(Z_STRVAL(pv), "return ", sizeof("return ") - 1);
-	memcpy(Z_STRVAL(pv) + sizeof("return ") - 1, expr, expr_len);
-	Z_STRVAL(pv)[Z_STRLEN(pv) - 1] = ';';
-	Z_STRVAL(pv)[Z_STRLEN(pv)] = '\0';
-	Z_TYPE_INFO(pv) = IS_STRING;
-
-	new_break.ops = zend_compile_string(&pv, "Conditional Breakpoint Code");
-
-	zval_dtor(&pv);
+	bp_code = zend_string_concat3(
+		"return ", sizeof("return ")-1, expr, expr_len, ";", sizeof(";")-1);
+	new_break.ops = zend_compile_string(bp_code, "Conditional Breakpoint Code");
+	zend_string_release(bp_code);
 
 	if (new_break.ops) {
 		brake = zend_hash_index_update_mem(&PHPDBG_G(bp)[PHPDBG_BREAK_COND], hash, &new_break, sizeof(phpdbg_breakcond_t));
@@ -860,7 +900,7 @@ static inline void phpdbg_create_conditional_break(phpdbg_breakcond_t *brake, co
 
 PHPDBG_API void phpdbg_set_breakpoint_expression(const char *expr, size_t expr_len) /* {{{ */
 {
-	zend_ulong expr_hash = zend_inline_hash_func(expr, expr_len);
+	zend_ulong expr_hash = zend_hash_func(expr, expr_len);
 	phpdbg_breakcond_t new_break;
 
 	if (!zend_hash_index_exists(&PHPDBG_G(bp)[PHPDBG_BREAK_COND], expr_hash)) {
@@ -879,7 +919,7 @@ PHPDBG_API void phpdbg_set_breakpoint_at(const phpdbg_param_t *param) /* {{{ */
 
 	if (param->next) {
 		condition = param->next;
-		hash = zend_inline_hash_func(condition->str, condition->len);
+		hash = zend_hash_func(condition->str, condition->len);
 
 		if (!zend_hash_index_exists(&PHPDBG_G(bp)[PHPDBG_BREAK_COND], hash)) {
 			phpdbg_create_conditional_break(&new_break, param, condition->str, condition->len, hash);
